#include "config.h"
#include "pt_cornell_1_2_1.h"

#include "tft_master.h"
#include "tft_gfx.h"

#include <stdlib.h>

#include "math.h"
#include "Goertzel.h"
#include "string.h"


//DAC and DDS macros
#define DAC_config_chan_A 0b0011000000000000
#define DAC_config_chan_B 0b1011000000000000
#define Fs 16000.0
#define two32 4294967296.0 // 2^32 


//FFT macros and variables
#define SAMPLING_RATE 4000.0 // declared in float, could change to 8192 so that we can use shift when dividing
#define N 1000 // could use 1024 or 512 so that we can use shift

volatile short adc_9;
volatile short dataArray0[N];
volatile short dataArray1[N];
volatile char dataArray0_flag = 1;
volatile char dataArray1_flag = 1;
volatile short *dataArray;
volatile int ind = 0;
volatile int arrayID = 0;
volatile unsigned int note_index;
volatile int cent_index = 10;

const float two_PI = M_PI * 2;
const float two_PI_div_N = M_PI / 500;


//display variables
static short baseX[12];
static short baseY[12];
static short tipX[11];
static short tipY[11];

static short markBaseX[22];
static short markBaseY[22];
static short markTipX[11];
static short markTipY[11];

#define outR 220
#define inR 202

#define bigR 220
#define smallR 202

//state machine variables
static int mode;
static int waiting;
static int state;

char buffer[60];

static struct pt pt_display, pt_calc, pt_uart, pt_input, pt_output, pt_DMA_output ;




static fail_count;



/**************************************DDS parts****************************************/
//SPI setup
volatile SpiChannel spiChn = SPI_CHANNEL2 ;	// the SPI channel to use
// for 60 MHz PB clock use divide-by-3
volatile int spiClkDiv = 2 ; // 15 MHz DAC clock

//fix point macros
typedef signed int fix16 ;
#define multfix16(a,b) ((fix16)(((( signed long long)(a))*(( signed long long)(b)))>>16)) //multiply two fixed 16:16
#define float2fix16(a) ((fix16)((a)*65536.0)) // 2^16
#define fix2float16(a) ((float)(a)/65536.0)
#define fix2int16(a)    ((int)((a)>>16))
#define int2fix16(a)    ((fix16)((a)<<16))
#define divfix16(a,b) ((fix16)((((signed long long)(a)<<16)/(b)))) 
#define sqrtfix16(a) (float2fix16(sqrt(fix2float16(a)))) 
#define absfix16(a) abs(a)
#define onefix16 0x00010000 // int2fix16(1)

// DDS sine table
#define sine_table_size 256
volatile fix16 sin_table[sine_table_size];

//function to build the sine table for DDS synthesizing
void buildSineTable(){
    // build the sine lookup table
    // scaled to produce values between 0 and 4096
    int t = 0;
    for (t = 0; t < sine_table_size; t++){
         sin_table[t] = (int)(2047*sin((float)t*6.283/(float)sine_table_size));
    } 
}

// actual scaled DAC 
volatile int DAC_data;  

volatile unsigned int phase_accum;
volatile unsigned int phase_incr;

/*********************************************end of DDS parts****************************************/


/*********************************************************FFT helper***********************************************************/

/*
 * Function to return the maximum index in the frequency array
 * float *arr, int n
 * 
 * Returns: unsigned int index
 */
unsigned int max_index(float *arr, int n){
    float maximum = *(arr);
    float temp;
    unsigned int i, index;
    index = 0;
    for (i = 1; i < n; i++){
        temp = *(arr + i);
        if (temp > maximum){
            maximum = temp;
            index = i;
        }
    }
    return index;
}

/* Helper function to convert the index to a string representing the note
 * Parameters:unsigned int index
 * 
 * Returns: string for note
 */
char *index2note(unsigned int index){
    char note[3];
    unsigned int index_note;
    if (index < 37) index_note = index % 12;
    else if (index == 37) index_note = -1;
    switch (index_note)
    {
        case -1:
            note[0] = '-';
            note[1] = '-';
            break;
        case 0:
            note[0] = 'C';
            note[1] = ' ';
            break;
        case 1:
            note[0] = 'C';
            note[1] = '#';
            break;
        case 2:
            note[0] = 'D';
            note[1] = ' ';
            break;
        case 3:
            note[0] = 'E';
            note[1] = 'b';
            break;
        case 4:
            note[0] = 'E';
            note[1] = ' ';
            break;
        case 5:
            note[0] = 'F';
            note[1] = ' ';
            break;
        case 6:
            note[0] = 'F';
            note[1] = '#';
            break;
        case 7:
            note[0] = 'G';
            note[1] = ' ';
            break;
        case 8:
            note[0] = 'A';
            note[1] = 'b';
            break;
        case 9:
            note[0] = 'A';
            note[1] = ' ';
            break;
        case 10:
            note[0] = 'B';
            note[1] = 'b';
            break;
        case 11:
            note[0] = 'B';
            note[1] = ' ';
            break;
        default:
            note[0] = '-';
            note[1] = '-';
            break;
    }
    note[2] = '\0';
    return note;
}

/* Function to perform the goertzel algorithm
 * Parameters:float target_freq
 * 
 * Returns: float output
 */
float Goertzel(float target_freq){
    int i, k;
    float w, coeff, Q0, Q1, Q2;
    float output, sq1, sq2, mult;
    //k = (int)(0.5 + N*target_freq/SAMPLING_RATE);
    //w = two_PI*k/N;
    k = (int)(0.5 + target_freq/4);
    w = k * two_PI_div_N;
    coeff = cos(w) * 2;
    Q0 = 0.0;
    Q1 = 0.0;
    Q2 = 0.0;
    for (i = 0; i < N; i++){
        Q0 = coeff * Q1 - Q2 + *(dataArray + i);
        Q2 = Q1;
        Q1 = Q0;
    }
    sq1 = Q1*Q1;
    sq2 = Q2*Q2;
    mult = Q1*Q2*coeff;
    output = sq1+sq2+mult;
    //output = Q1 * Q1 + Q2 * Q2 - Q1 * Q2 * coeff;
    return output;
}

/************************************************end of FFT helper***********************************************************/


/****************************************************DISPLAY SETUP functions***********************************************/
/* Draw circle with customized thickness
 * Parameters:
 *      x,y - location of center
 *      r   - radius
 *      size - thickness of circle
 *      color
 * Returns: Nothing
 */

void drawCircle(short x, short y, short r, short size, short color){
    while (size > 0){
        tft_drawCircle(x, y, (r+size), color); 
        size --;
    }
}

/* set up the three vertices for each triangle. 
 * Trigonometric Identities are used to calculate the point indices based on the sides and angles, 
 * and these calculated indices are stored in arrays markBaseX[22], markBaseY[22], markTipX[11],markTipY[1
 * Parameters: None
 * 
 * Returns: Nothing
 */
void markSetup(){
    int i;
    float rad;
    float step = 6.6/180.0*M_PI;
    float markSep = 0.5/180.0*M_PI;
    rad = 0 - step * 5.0;
    for (i = 0; i < 11; i ++){
        markTipX[i] = (short)(120 + inR*sin(rad));
        markTipY[i] = (short)(270 - inR*cos(rad));
        markBaseX[2*i] = (short)(120 + outR*sin(rad-markSep));
        markBaseX[2*i+1] = (short)(120 + outR*sin(rad+markSep));
        markBaseY[2*i] = (short)(270 - outR*cos(rad-markSep));
        markBaseY[2*i+1] = (short)(270 - outR*cos(rad+markSep));
        rad = rad + step;
    }  
}

/*  draw the marks using the point locations calculated using markSetup()
 * Parameters: None
 * 
 * Returns: None
 */
void drawMarks(){
    int i;
    int j;
    int m = -50;
    for (i = 0; i < 11; i++){
        tft_fillTriangle(markTipX[i], markTipY[i], markBaseX[2*i], markBaseY[2*i], markBaseX[2*i+1], markBaseY[2*i+1], ILI9340_WHITE);
        if (i < 5 ){
            tft_setCursor(markBaseX[2*i], (markBaseY[2*i] - 20));
            tft_setTextSize(1);
        }
        else if (i > 5){
            tft_setCursor((markBaseX[2*i]-7), (markBaseY[2*i] - 20));
            tft_setTextSize(1);
        }
        else{
            tft_setCursor(markBaseX[2*i]-2, (markBaseY[2*i] - 23));
            tft_setTextSize(2);
        }
        tft_setTextColor(ILI9340_WHITE); 
        sprintf(buffer,"%d", m);
        tft_writeString(buffer);
        m = m + 10;
    }
}

/*  calculates the vertices of the triangles that are used to represent the arrows pointing at the meter.
 * Parameters: None
 * 
 * Returns: None
 */
void arrowIndexSetup(){
    int i;
    float rad;
    float step = 6.6/180.0*M_PI;
    rad = 0 - step * 5.5;
    for (i = 0; i < 12; i ++){
        baseX[i] = (short)(120 + smallR*sin(rad));
        baseY[i] = (short)(270 - smallR*cos(rad));
        rad = rad + step;
    }  
    rad = 0 - step * 5.0;
    for (i = 0; i < 11; i ++){
        tipX[i] = (short)(120 + bigR*sin(rad));
        tipY[i] = (short)(270 - bigR*cos(rad));
        rad = rad + step;
    } 
}

/* draw the arrow based on the value of cent_index, 
 * which indicates how accurate the input sound frequency is compared to its nearest standard note frequency.
 * Parameters: None
 * 
 * Returns: None
 */
void drawArrow(){
    //tft_fillCircle(120, 270, 220, ILI9340_BLACK);
    if (cent_index == 10) tft_fillTriangle(tipX[5], tipY[5], baseX[5], baseY[5], baseX[6], baseY[6], ILI9340_GREEN);
    else if ((cent_index < 2) || (cent_index > 8)){
        tft_fillTriangle(tipX[cent_index], tipY[cent_index], baseX[cent_index], baseY[cent_index], baseX[cent_index+1], baseY[cent_index+1], ILI9340_RED);
        //tft_fillTriangle(120, 270, baseX[cent_index], baseY[cent_index], baseX[cent_index+1], baseY[cent_index+1], ILI9340_YELLOW);
    }
    else{
        tft_fillTriangle(tipX[cent_index+1], tipY[cent_index+1], baseX[cent_index+1], baseY[cent_index+1], baseX[cent_index+2], baseY[cent_index+2], ILI9340_YELLOW);
        //tft_fillTriangle(120, 270, baseX[cent_index+1], baseY[cent_index+1], baseX[cent_index+2], baseY[cent_index+2], ILI9340_RED);
    }
    //tft_fillCircle(120, 270, 202, ILI9340_BLACK);
}

/* clear all the previously drawn arrow before drawing a new one
 * Parameters: None
 * 
 * Returns: None
 */
void clearArrow(){
    int i;
    for (i = 0; i < 11; i ++){
        tft_fillTriangle(tipX[i], tipY[i], baseX[i], baseY[i], baseX[i+1], baseY[i+1], ILI9340_BLACK);
    }
}

/* integrated setup helper function
 * Parameters: None
 * 
 * Returns: None
 */
void staticDisplaySetup(){
    clearArrow();
    drawCircle(120, 270, 220, 5,ILI9340_WHITE); 
    drawMarks();
}


// The three functions below are the previously used ones for the bar line user interface attempt
void BarDisplaySetup(){
    tft_drawRect(50, 200, 15, 30, ILI9340_WHITE);
    tft_drawRect(67, 180, 15, 50, ILI9340_WHITE);
    tft_drawRect(84, 160, 15, 70, ILI9340_WHITE);
    tft_drawRect(101, 140, 15, 90, ILI9340_WHITE);
    tft_drawRect(118, 120, 15, 110, ILI9340_WHITE);
    tft_drawRect(187, 120, 15, 110, ILI9340_WHITE);
    tft_drawRect(204, 140, 15, 90, ILI9340_WHITE);
    tft_drawRect(221, 160, 15, 70, ILI9340_WHITE);
    tft_drawRect(238, 180, 15, 50, ILI9340_WHITE);
    tft_drawRect(255, 200, 15, 30, ILI9340_WHITE);
}

void clearBars(){
    tft_fillRect(50, 200, 15, 30, ILI9340_BLACK);
    tft_fillRect(67, 180, 15, 50, ILI9340_BLACK);
    tft_fillRect(84, 160, 15, 70, ILI9340_BLACK);
    tft_fillRect(101, 140, 15, 90, ILI9340_BLACK);
    tft_fillRect(118, 120, 15, 110, ILI9340_BLACK);
    tft_fillRect(187, 120, 15, 110, ILI9340_BLACK);
    tft_fillRect(204, 140, 15, 90, ILI9340_BLACK);
    tft_fillRect(221, 160, 15, 70, ILI9340_BLACK);
    tft_fillRect(238, 180, 15, 50, ILI9340_BLACK);
    tft_fillRect(255, 200, 15, 30, ILI9340_BLACK);
}

void fillBars(){
    clearBars();
    BarDisplaySetup();
    if (cent_index < 10){
            if (cent_index < 5){
            tft_fillRect(118, 120, 15, 110, ILI9340_WHITE);
            if (cent_index < 4){
                tft_fillRect(101, 140, 15, 90, ILI9340_WHITE);
            }
            if (cent_index < 3){
                tft_fillRect(84, 160, 15, 70, ILI9340_WHITE);
            }
            if (cent_index < 2){
                tft_fillRect(67, 180, 15, 50, ILI9340_WHITE);
            }
            if (cent_index < 1){
                tft_fillRect(50, 200, 15, 30, ILI9340_WHITE);
            }
        }
        else{
            tft_fillRect(187, 120, 15, 110, ILI9340_WHITE);
            if (cent_index > 5){
                tft_fillRect(204, 140, 15, 90, ILI9340_WHITE);
            }
            if (cent_index > 6){
                tft_fillRect(221, 160, 15, 70, ILI9340_WHITE);
            }
            if (cent_index > 7){
                tft_fillRect(238, 180, 15, 50, ILI9340_WHITE);
            }
            if (cent_index > 8){
                tft_fillRect(255, 200, 15, 30, ILI9340_WHITE);
            }
        }
    }
}

/*********************************END OF DISPLAY FUNCTIONS************************************************/


/*********************************Timer 3 ISR******************************************************/

void __ISR(_TIMER_3_VECTOR, ipl3) Timer3Handler(void)
{
    mT3ClearIntFlag();  
    mPORTBSetBits(BIT_4);     // end SPI transaction from last interrupt cycle
    
    if (mode == 2 && state == 2){
        mPORTBClearBits(BIT_4); // start transaction
        phase_accum += phase_incr;
        DAC_data = sin_table[phase_accum>>24];
        WriteSPI2( DAC_config_chan_A | DAC_data+2048);  
    }
} 




/******************Timer 2 ISR******************************************************/
void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void)
{
    adc_9 = ReadADC10(0);   // read the result of channel 9 conversion from the idle buffer
    AcquireADC10(); // not needed if ADC_AUTO_SAMPLING_ON below
    if (arrayID == 0){
        *(dataArray0 + ind) = (adc_9 > 0x3FC) ? 0 : (0x3FF - adc_9);
        if (ind == N){
            ind = 0;
            arrayID = 1;
            dataArray0_flag = 1;
        }
        else if (ind == 0) dataArray0_flag = 0;
        ind ++;
    }
    else if (arrayID == 1){
        *(dataArray1 + ind) = (adc_9 > 0x3FC) ? 0 : (0x3FF - adc_9);
        if (ind == N){
            ind = 0;
            arrayID = 0;
            dataArray1_flag = 1;
        }
        else if (ind == 0) dataArray1_flag = 0;
        ind ++;
    }
    
    mT2ClearIntFlag();    // clear the timer interrupt flag
}


/**********************thread for calc*******************************/

static PT_THREAD (protothread_calc(struct pt *pt))
{
    PT_BEGIN(pt);

    float results[36];
    int i;
    unsigned int prev_note_index;
    unsigned int temp_note_index;
    float freq2display, freq_step_down;
    float max_result = 0.0;
    float temp_freq = 0.0;
    float cent_results[10];
    double thres;
    int start_time = 0;
    while(1) {
        if (mode == 1){
            PT_YIELD_TIME_msec(5);
            start_time = PT_GET_TIME();
            dataArray = (arrayID) ? dataArray0 : dataArray1;
            if ((arrayID == 0 && dataArray0_flag == 1) || (arrayID == 1 && dataArray1_flag == 1)){
                for (i = 0; i < 36; i++){
                    temp_freq = Goertzel(FREQ_NOTES[i]);
                    if (temp_freq > max_result){
                        max_result = temp_freq;
                        temp_note_index = i;
                    }
                }
                thres = 100000000.0;
                if (max_result > thres){
                    prev_note_index = temp_note_index;

                    freq2display = FREQ_NOTES[note_index];
                    freq_step_down = FREQ_NOTES[note_index - 1] * FIFTY_CENTS_STEP;
                    cent_index = 10;
                    for (i = 0; i < 10; i++){
                        freq_step_down *= TEN_CENTS_STEP;
                        if (Goertzel(freq_step_down) > max_result){
                            cent_index = i;
                        }
                    }
            
                    note_index = prev_note_index;
                }
            }
        }
        PT_YIELD_TIME_msec(300);
      } // END WHILE(1)
  PT_END(pt);
} 




/*****************************UART******************************************/
// The serial interface
static char cmd[16]; 
static int pos;

/*clear the string cmd*/
void clearCmd(){
    int i;
    for (i = 0; i < strlen(cmd); i ++){
        cmd[i] = 0;
    }
}

static int key;

/* Thread for user input acquisition and state machine transition*/
static PT_THREAD (protothread_uart(struct pt *pt))
{
    PT_BEGIN(pt);  
        while(1) {     
            switch(state){       
                //idle state
                case 0:
                    fail_count = 0;
                    tft_fillScreen(ILI9340_BLACK);
                    sprintf(PT_send_buffer,"Please select the tuning mode. '1' for auto, '2' for manual>\r\n");           
                    PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                    PT_SPAWN(pt, &pt_input, PT_GetSerialBuffer(&pt_input) );
                    sscanf(PT_term_buffer, "%d", &mode);
                    switch (mode){
                        case 1:
                            state = 2; //switch to wait state
                            IEC0SET = 0x02000;
                            tft_fillScreen(ILI9340_BLACK);
                            break;
                        case 2:
                            state = 1; //switch to manual state
                            waiting = 0;
                            tft_fillScreen(ILI9340_BLACK);
                            break;
                        default:
                            sprintf(PT_send_buffer,"Oops! Unknown error has occurred, please try again!\r\n");           
                            PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                            break;
                    }
                    break;
                //input acquisition state          
                case 1:
                    sprintf(PT_send_buffer,"Input a node like 'C 4' or 'C# 4'>");           
                    PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                    PT_SPAWN(pt, &pt_input, PT_GetSerialBuffer(&pt_input) );
                    sscanf(PT_term_buffer, "%s %d", cmd, &pos);
                    state = 2;
                    switch(cmd[0]){
                        case 'C': 
                            key = 0;  
                            break;
                        case 'D':
                            key = 2; 
                            break;
                        case 'E':
                            key = 4; 
                            break;
                        case 'F': 
                            key = 5; 
                            break;
                        case 'G':
                            key = 7; 
                            break;
                        case 'A':
                            key = 9; 
                            break;
                        case 'B':
                            key = 11; 
                            break;
                        default:
                            if (fail_count > 5){
                                sprintf(PT_send_buffer,"Crashed!\r\n");           
                                PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                                fail_count = 0;
                            }
                            else{
                                sprintf(PT_send_buffer,"Oops! Unknown error has occurred, please try again!\r\n");           
                                PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                                fail_count ++;
                                state = 1;
                            }
                            break;
                    }
                    if (cmd[1] == '#'){
                        key = key + 1;
                    }
                    
                    tft_fillRoundRect(20,40, 200, 100, 1, ILI9340_BLACK);
                    tft_setCursor(20, 40);
                    tft_setTextColor(ILI9340_YELLOW); tft_setTextSize(2);       
                    sprintf(buffer,"%s %d", cmd, pos);
                    tft_writeString(buffer);
                    
                    key = key + 12 * (pos-1);
                    phase_incr = FREQ_MANUAL[key]*two32/Fs;
                    phase_accum = 0;
                    IEC0SET = 0x08000;
                    waiting = 1;
                    break;
                // running state
                case 2:
                    sprintf(PT_send_buffer,"type 'q' to stop>");           
                     PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                     PT_SPAWN(pt, &pt_input, PT_GetSerialBuffer(&pt_input) );
                     if (PT_term_buffer[0] == 'q'){
                         if (mode == 1){
                            IEC0CLR = 0x02000; 
                         }
                         else{
                            IEC0CLR = 0x08000;   
                         }
                         state = 0;
                         mode = 3;
                         clearCmd();
                         pos = 0;
                     }
                     else{
                        sprintf(PT_send_buffer,"Oops! Unknown error has occurred, please try again!\r\n");           
                        PT_SPAWN(pt, &pt_DMA_output, PT_DMA_PutSerialBuffer(&pt_DMA_output) );
                     }
                     break;
                     
            }
      } // END WHILE(1)
  PT_END(pt);
}


/* Thread for TFT display
 */

static PT_THREAD (protothread_display(struct pt *pt))
{
    PT_BEGIN(pt);
    //char note[2];
    while(1) {
//        fillBars();
        switch(mode){
            //idle state
            case 0:
                tft_setCursor(20, 100);
                tft_setTextColor(ILI9340_YELLOW); tft_setTextSize(1);       
                tft_writeString("Hello, this is a tuner.\0");
                break;
            //input acquisition state
            case 1:
                staticDisplaySetup();
                drawArrow();
                tft_fillRoundRect(95, 150, 100, 100, 1, ILI9340_BLACK);
                tft_setCursor(95, 150);
                if (cent_index == 10) tft_setTextColor(ILI9340_GREEN); 
                else if ((cent_index < 2) || (cent_index > 8)) tft_setTextColor(ILI9340_RED);
                else tft_setTextColor(ILI9340_YELLOW);
                tft_setTextSize(5);       
                tft_writeString(index2note(note_index));
                break;

            //running state
             case 2:
                if (waiting == 0){
                    tft_fillRoundRect(20,20, 200, 100, 1, ILI9340_BLACK);
                    tft_setCursor(20, 20);
                    tft_setTextColor(ILI9340_YELLOW); tft_setTextSize(1);       
                    tft_writeString("Manual mode, waiting for commands.\0");
                }
                else{
                    tft_fillRoundRect(20,20, 200, 20, 1, ILI9340_BLACK);
                    tft_setCursor(20, 20);
                    tft_setTextColor(ILI9340_YELLOW); tft_setTextSize(2);       
                    tft_writeString("Now playing:\0");

                }
                break;
        }
        PT_YIELD_TIME_msec(1000);

      } // END WHILE(1)
  PT_END(pt);
} 




int main(int argc, char** argv) {
    
    /*************** calibrate the internal clock*****************************/
	SYSKEY = 0x0; // ensure OSCCON is locked
    SYSKEY = 0xAA996655; // Write Key1 to SYSKEY
    SYSKEY = 0x556699AA; // Write Key2 to SYSKEY
    // OSCCON is now unlocked
    // make the desired change
    OSCTUN = 0b111100; // tune the rc oscillator
    // Relock the SYSKEY
    SYSKEY = 0x0; // Write any value other than Key1 or Key2
    /******************** OSCCON is relocked*****************************/
    
    
    
    ANSELA = 0; ANSELB = 0; 
    PT_setup();
    INTEnableSystemMultiVectoredInt();    // setup system wide interrupts
    
    //timer 2 for FFT
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 10000);
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
    IEC0CLR = 0x02000;
    
    /************************added part for DDS*************************************/
    //timer 3 for DDS
    OpenTimer3(T3_ON | T3_SOURCE_INT | T3_PS_1_1, 2500);    // 2500 is 16 ksamples/sec at 40 MHz clock
    ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_2);    // set up the timer interrupt with a priority of 2
    mT3ClearIntFlag(); // and clear the interrupt flag
    
    
    /*SPI setup SCK2 is pin 26,  SDO2 (MOSI) is in PPS output group 2, could be connected to RB5 which is pin 14*/
    PPSOutput(2, RPB5, SDO2);
    mPORTBSetPinsDigitalOut(BIT_4);    // control CS for DAC
    mPORTBSetBits(BIT_4);
    SpiChnOpen(spiChn, SPI_OPEN_ON | SPI_OPEN_MODE16 | SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV , spiClkDiv);
    
    buildSineTable();
    
/********************configure and enable the ADC**************************************************************/
    CloseADC10();    // ensure the ADC is off before setting the configuration

    /*define setup parameters for OpenADC10
    Turn module on | ouput in integer | trigger mode auto | enable autosample
    ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();*/
    #define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_OFF //

    /*define setup parameters for OpenADC10
    ADC ref external  | disable offset test | disable scan mode | do 1 sample | use single buf | alternate mode off*/
    #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
    
    /* Define setup parameters for OpenADC10
    use peripherial bus clock | set sample time | set ADC clock divider
    ADC_CONV_CLK_Tcy2 means divide CLK_PB by 2 (max speed)
    ADC_SAMPLE_TIME_5 seems to work with a source resistance < 1kohm*/
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_5 | ADC_CONV_CLK_Tcy2 //ADC_SAMPLE_TIME_15| ADC_CONV_CLK_Tcy2

    /*define setup parameters for OpenADC10
     set AN11 and  as analog inputs*/
    #define PARAM4    ENABLE_AN11_ANA // pin 24

    /*define setup parameters for OpenADC10
    do not assign channels to scan*/
    #define PARAM5    SKIP_SCAN_ALL

    /* use ground as neg ref for A | use AN11 for input A     
    configure to sample AN11 */
    SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN11 ); // configure to sample AN4 
    OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using the parameters defined above

    EnableADC10(); // Enable the ADC
    
    // init the threads
    PT_INIT(&pt_display);
    PT_INIT(&pt_calc);
    PT_INIT(&pt_uart);
    
    // init the display
    tft_init_hw();
    tft_begin();
    tft_fillScreen(ILI9340_BLACK);
    //240x320 vertical display
    tft_setRotation(0); // Use tft_setRotation(1) for 320x240

    ind = 0;
    markSetup();
    arrowIndexSetup();
    fail_count = 0;
    state = 0;
    mode = 0;
    waiting = 0;
    clearCmd();
    pos = 0;
    
    // round-robin scheduler for threads
    while (1){
        PT_SCHEDULE(protothread_display(&pt_display));
        PT_SCHEDULE(protothread_calc(&pt_calc));
        PT_SCHEDULE(protothread_uart(&pt_uart)); 
    }
}